<?php

namespace Drupal\advanced_email_validation;

use Drupal\Core\Config\ConfigFactoryInterface;
use EmailValidator\EmailValidator;

/**
 * Email validator service.
 */
class EmailValidatorWrapperService {

  const ADVANCED_EMAIL_VALIDATION_VALID = 'valid';

  const ADVANCED_EMAIL_VALIDATION_INVALID = 'invalid';

  const ADVANCED_EMAIL_VALIDATION_DISPOSABLE = 'disposable';

  const ADVANCED_EMAIL_VALIDATION_FREE = 'free';

  const ADVANCED_EMAIL_VALIDATION_BANNED = 'banned';

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Constructs an EmailValidator object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $config = $config_factory->get('advanced_email_validation.settings');
    $rules = $config->get('rules');
    $this->config = [
      'checkMxRecords' => $rules[self::ADVANCED_EMAIL_VALIDATION_INVALID] !== '0',
      'checkBannedListedEmail' => $rules[self::ADVANCED_EMAIL_VALIDATION_BANNED] !== '0',
      'checkDisposableEmail' => $rules[self::ADVANCED_EMAIL_VALIDATION_DISPOSABLE] !== '0',
      'checkFreeEmail' => $rules[self::ADVANCED_EMAIL_VALIDATION_FREE] !== '0',
      'bannedList' => $config->get('banned_list'),
      'disposableList' => $config->get('disposable_list'),
      'freeList' => $config->get('free_list'),
    ];
  }

  /**
   * Validate an email address.
   *
   * Uses the configuration set in the configuration UI by default.
   *
   * @param string $email
   *   The email address to be validated
   * @param array $configOverrides
   *   Optional configuration overrides
   *   - checkMxRecords: boolean, test the email is from a valid provider
   *   - checkBannedListedEmail: boolean, test the email isn't on your banned
   *       list
   *   - checkDisposableEmail: boolean, test the email isn't from a disposable
   *       address provider like mailinator
   *   - checkFreeEmail: boolean, test the email isn't from a free address
   *       provider like gmail
   *   - bannedList: array, list of domains to ban
   *   - disposableList: array, list of additional disposable domains
   *   - freeList: array, list of additional free domains
   *
   * @return string
   */
  public function validate(string $email, array $configOverrides = []): string {
    $config = array_merge($this->config, $configOverrides);
    $emailValidator = new EmailValidator($config);
    $result = $emailValidator->validate($email);

    if ($result === TRUE) {
      return self::ADVANCED_EMAIL_VALIDATION_VALID;
    }
    $reason = $emailValidator->getErrorReason();
    switch ($reason) {
      case 'Domain does not accept email' :
        return self::ADVANCED_EMAIL_VALIDATION_INVALID;
      case 'Domain is used by free email providers' :
        return self::ADVANCED_EMAIL_VALIDATION_FREE;
      case 'Domain is used by disposable email providers' :
        return self::ADVANCED_EMAIL_VALIDATION_DISPOSABLE;
      case 'Domain is banned' :
        return self::ADVANCED_EMAIL_VALIDATION_BANNED;
    }
    return self::ADVANCED_EMAIL_VALIDATION_VALID;
  }

}
